module ChaseImports (
 ChaseName,
 chaseWith,
 chaseFile 
) where

import Monad
import IO



--- Any kind of module name --------------------------------------------------

type ChaseName = String



--- The recursiver chaser ----------------------------------------------------

chaseWith :: [FilePath]						  -- path 
          -> [ChaseName] 					  -- todo
	  -> [ChaseName] 				          -- done
	  -> accu     						  -- initial
          -> ([FilePath] -> ChaseName -> IO (Either cu String))   -- parse
	  -> (cu -> [ChaseName]) 				  -- imports
	  -> (ChaseName -> [ChaseName] -> cu -> accu -> IO accu)  -- on module
	  -> (ChaseName -> accu -> IO accu) 			  -- on missing
	  -> IO accu

chaseWith dirs todo done accu parseFile getImports onModule onMissing
 = chase todo done accu
   where
    chase [] done accu
      = do errLn "Import chasing complete."
           return accu
    chase (m:ms) done accu
      | m `elem` done = chase ms done accu
      | otherwise     = processFile `mplus` skipFile
     where
       processFile 
         = do parse_result <- parseFile dirs m
	      case parse_result of
	        Left pin  
		  -> let is = getImports pin
                     in do accu' <- onModule m is pin accu
		           chase (ms++is) (m:done) accu'
	        Right msg 
		  -> do errLn ("Failed to parse "++m++": "++msg)
		        accu' <- onMissing m accu
		        chase ms (m:done) accu'     
       skipFile    
         = do errLn ("Could not find module "++m++": skipping.")
	      accu' <- onMissing m accu
	      chase ms (m:done) accu'



--- Read a file from a number of possible directories ------------------------
	      
chaseFile dirs basename exts
  = do results <- mapM tryReadFile fnames
       case dropWhile hasFailed results of
         ((Right (fc,fn)):_) 
	     -> errLn ("Read file: "++fn) >> return fc
	 _   -> errLn ("Could not find file: "++basename) >> mzero                  
    where 
      fnames = [d++'/':basename++'.':e | d <- dirs, e <- exts]
      hasFailed (Left _) = True
      hasFailed _        = False
      tryReadFile fn = try ( readFile fn >>= \fc -> return (fc,fn) )
      


--- Progress and error messages ----------------------------------------------

errLn str = hPutStrLn stderr str
