{-----------------------------------------------------------------------------

	         A LIBRARY OF FUNCTIONAL STRATEGY COMBINATORS

			          StrategyLib

                   Ralf Laemmel                Joost Visser
               CWI & VU, Amsterdam         CWI & SIG, Amsterdam

This module is part of a library of functional strategy combinators,
including combinators for generic traversal. This module defines
combinators that iterate until some kind of fixpoint is reached.

-----------------------------------------------------------------------------} 

module FixpointTheme where

import StrategyPrelude
import FlowTheme
import TraversalTheme
import Monad


--- Fixpoint iteration -------------------------------------------------------

-- Exhaustive repeated application
repeatTP        :: MonadPlus m => TP m -> TP m
repeatTP s      =  tryTP (seqTP s (repeatTP s))


--- Fixpoint traversal -------------------------------------------------------

reduce 		:: MonadPlus m => TP m -> TP m
reduce s 	=  repeatTP (someTP (reduce s) `choiceTP` s)

outermost  	:: MonadPlus m => TP m -> TP m
outermost s	=  repeatTP (once_tdTP s)

-- Naive innermost
innermost'	:: MonadPlus m => TP m -> TP m
innermost' s	=  repeatTP (once_buTP s)

-- More efficient innermost
innermost	:: MonadPlus m => TP m -> TP m
innermost s	=  allTP (innermost s) 
                   `seqTP` 
                   (tryTP (s `seqTP` (innermost s)))

------------------------------------------------------------------------------

