{-----------------------------------------------------------------------------
 
	        A LIBRARY OF FUNCTIONAL STRATEGY COMBINATORS
 
		                  StrategyLib
 
                   Ralf Laemmel                Joost Visser
               CWI & VU, Amsterdam            CWI, Amsterdam
 
This module is part of a library of functional strategy combinators,
including combinators for generic traversal. In this module, we define
path combinator to constrain selection and transformation of nodes or
subtrees by path conditions.
 
-----------------------------------------------------------------------------} 



module PathTheme where

import StrategyPrelude
import OverloadingTheme
import FlowTheme
import TraversalTheme
import Monad



--- Select/transform a node below a node where a condition holds -------------

-- We find the top-most node which admits selection/transformation
-- below the top-most node which meets the condition. Thus, the
-- distance between guard and application node is minimized.

belowS, beloweqS :: (MonadPlus m, Strategy s m, StrategyPlus s m)
                 => s m -> TU () m -> s m
s `belowS`   s'  =  (oneS s) `beloweqS` s'
s `beloweqS` s'  =  once_td (ifthenS s' (once_td s))

-- Type-specialised versions
belowTP, beloweqTP :: MonadPlus m => TP m -> TU () m -> TP m
belowTP            =  belowS
beloweqTP          =  beloweqS



--- Select/transform a node above a node where a condition holds -------------

aboveS, aboveeqS :: (MonadPlus m, Strategy s m, StrategyPlus s m)
                 => s m -> TU () m -> s m
s `aboveS`   s'  =  s `aboveeqS` (oneTU s')
s `aboveeqS` s'  =  once_bu (ifthenS (once_tdTU s') s)


-- Type-specialised versions
aboveTP, aboveeqTP :: MonadPlus m => TP m -> TU () m -> TP m
aboveTP            =  aboveS
aboveeqTP          =  aboveeqS

------------------------------------------------------------------------------

