{-----------------------------------------------------------------------------

	        A LIBRARY OF FUNCTIONAL STRATEGY COMBINATORS
				   
 		                  StrategyLib

                   Ralf Laemmel                Joost Visser
               CWI & VU, Amsterdam         CWI & SIG, Amsterdam

This module is part of a library of functional strategy combinators,
including combinators for generic traversal. This module is basically
a wrapper for the strategy primitives plus some extra basic strategy
combinators that can be defined immediately in terms of the primitive
ones.

-----------------------------------------------------------------------------}

module StrategyPrelude (
  module StrategyPrimitives,
  module StrategyPrelude
) where

import StrategyPrimitives
import Monad
import Monoid



--- Userful defaults for strategy update -------------------------------------

idTP		:: Monad m => TP m
idTP		=  paraTP return

failTP          :: MonadPlus m => TP m
failTP          =  paraTP (const mzero)

failTU          :: MonadPlus m => TU a m
failTU          =  paraTU (const mzero)

constTU         :: Monad m => a -> TU a m
constTU a       =  paraTU (const (return a))

compTU         :: Monad m => m a -> TU a m
compTU a       =  paraTU (const a)



--- Lift a function to a strategy type with failure as default ---------------

monoTP          :: (Term a, MonadPlus m) => (a -> m a) -> TP m
monoTP          =  adhocTP failTP

monoTU 		:: (Term a, MonadPlus m) => (a -> m b) -> TU b m
monoTU  	=  adhocTU failTU



--- Function composition of monomorphic function and TU strategy  ------------

dotTU        :: Monad m => (a -> b) -> TU a m -> TU b m
dotTU f s    =  s `passTU` (constTU . f)



--- Combining two TU strategies with a binary combinator ---------------------
--- (Was formally called comb but this clashs with other developments) -------

op2TU :: Monad m => (a -> b -> c) -> TU a m -> TU b m -> TU c m
op2TU o s s' =  s  `passTU` \a ->
                s' `passTU` \b ->
                constTU (o a b)



--- Reduce a strategy's performance to its effects, e.g., success vs. failure

voidTP    :: Monad m => TP m -> TU () m
voidTP s  =  s `seqTU` constTU ()

voidTU    :: Monad m => TU u m -> TU () m
voidTU s  =  s `passTU` \_ -> constTU ()



--- Shape combinators --------------------------------------------------------

con, com :: MonadPlus m => TP m
con      =  allTP failTP	-- test for constant term
com      =  oneTP idTP  	-- test for compound term



------------------------------------------------------------------------------